<?php
/**
 * 通过curl的方式发送HTTP请求，包括Get、Post
 *
 * @author zhaoxianlie
 */
class HttpClient {

    /**
     * 发送一个Post请求
     * @param $url
     * @param $data
     * @param bool $cookie
     * @return array
     */
    function post($url, $data, $cookie = false) {
        return $this->exec($url, $data, $cookie);
    }

    /**
     * 发送一个Get请求
     * @param $url
     * @param $cookie
     * @return array
     */
    function get($url, $cookie) {
        return $this->exec($url, false, $cookie, false);
    }

    /**
     * 返回array(cookie=>, body=>)
     * @param $url
     * @param $data
     * @param bool $cookie
     * @param bool $isPost
     * @return array
     */
    private function exec($url, $data, $cookie = false, $isPost = true) {
        $dataStr = "";
        if($data && is_array($data)) {
            foreach($data as $key => $value) {
                $dataStr .= "$key=$value&";
            }
        }

        // 启动一个CURL会话
        $curl = curl_init();
        // 要访问的地址
        curl_setopt($curl, CURLOPT_URL, $url);
        // 对认证证书来源的检查
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        // 从证书中检查SSL加密算法是否存在
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 1);
        // 模拟用户使用的浏览器
        curl_setopt($curl, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
        // 使用自动跳转
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
        $referer = "https://mp.weixin.qq.com/cgi-bin/singlesendpage"
            . "?t=message/send"
            . "&action=index"
            . "&tofakeid={$data['tofakeid']}"
            . "&token={$data['token']}"
            . "&lang=zh_CN";
        // 设置refer地址;
        curl_setopt($curl, CURLOPT_HTTPHEADER, array("Referer:$referer"));

        if($isPost) {
            // 发送一个常规的Post请求
            curl_setopt($curl, CURLOPT_POST, 0);
            // Post提交的数据包
            curl_setopt($curl, CURLOPT_POSTFIELDS, $dataStr);
        }

        // 设置超时限制防止死循环
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        // 显示返回的Header区域内容
        curl_setopt($curl, CURLOPT_HEADER, 1);
        // 获取的信息以文件流的形式返回
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);

        // 设置cookie
        if($cookie) {
            curl_setopt($curl, CURLOPT_COOKIE, $cookie);
        }

        // 执行操作
        $tmpInfo = curl_exec($curl);
        if (curl_errno($curl)) {
            // 捕抓异常
           echo 'Errno'. curl_error($curl);
           return;
        }

        // 关闭CURL会话
        curl_close($curl);

        // 解析HTTP数据流
        list($header, $body) = explode("\r\n\r\n", $tmpInfo);

        if(!$cookie) {
            // 解析COOKIE
            $cookie = "";
            preg_match_all("/set\-cookie: (.*)/i", $header, $matches);
            if(count($matches == 2)) {
                foreach($matches[1] as $each) {
                    $cookie .= trim($each). ";";
                }
            }
        }

        return array("cookie" => $cookie, "body" => trim($body));
    }
}
